# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/API_DataSource.ipynb (unless otherwise specified).

__all__ = ["DataSource"]

# Cell

from typing import *

# Internal Cell

import os

import pandas as pd
from fastcore.foundation import patch
from datetime import datetime, timedelta
from pathlib import Path
import requests

from ..logger import get_logger, set_level
from ..helper import (
    get_data,
    post_data,
    delete_data,
    add_ready_column,
    generate_df,
    get_tag_str,
)

from .client import Client
from .progress_status import ProgressStatus
from .model import Model
from .prediction import Prediction

# Internal Cell

logger = get_logger(__name__)

# Cell


class DataSource:
    """A class for encapsulating the data from sources like CSV files, databases, or AWS S3 buckets.

    The DataSource class is automatically instantiated by calling either the `csv`, `db` or the `s3` static methods of a DataSource class.
    Currently, it is the only way to instantiate this class.

    Currently, we support reading and pushing the data from:

    - a local CSV file,

    - a MySql database, and

    - an AWS S3 bucket in the Parquet file format.

    We plan to add other databases and storage mediums in the future.

    For reading and pushing from a local CSV file, a relative or an absolute path to the target CSV file
    is required.

    And for establishing the connection to the MySql database, parameters like host, port,
    database name, table name needs are required.

    And for establishing a connection to the S3 bucket, URI to the target parquet file
    is required.

    In case if access to the database requires authentication, the required username and
    password for the database are automatically read from either the environment variables
    **AIRT_CLIENT_DB_USERNAME** and **AIRT_CLIENT_DB_PASSWORD** or from the username and password
    parameters that are passed to the `db` methods of the DataSource class.

    All the function calls to the library are asynchronous and they return immediately.
    To manage completion, all methods inside the returned object will return a status object
    and a method to display an interactive progress bar that can be called to check the progress.

    Below are code examples for accessing the above methods:

    An example to check for the status flag of s3 connection:

    ```python
    data_source_s3 = DataSource.s3(
        uri="s3://bucket/events.parquet"
    )
    status = data_source_s3.pull()
    status.is_ready()
    ```

    An example to display an interactive progress bar of s3 connection:

    ```python
    data_source_s3 = DataSource.s3(
        uri="s3://bucket/events.parquet"
    )
    data_source_s3.pull().progress_bar()
    ```

    """

    def __init__(
        self, data_id: int, data_type: Optional[str] = None, csv_pull_params: dict = {}
    ):
        """Constructs a new DataSource instance.

        Warning:
            Do not construct this object directly by calling the constructor, please use `DataSource.db` or `DataSource.s3` function instead.

        Args:
            data_id: ID of the data source in the airt service.
            data_type: The type of the data source in the airt service.
            csv_pull_params: Additional parameters for processing the csv file in airt service. This includes
                deduplicate_data, index_column, sort_by, blocksize, kwargs_json.
        """
        self.id = data_id
        self.data_type = data_type
        self.csv_pull_params = csv_pull_params

    @staticmethod
    def s3(
        *,
        uri: str,
        access_key: Optional[str] = None,
        secret_key: Optional[str] = None,
        tag: Optional[str] = None,
    ) -> "DataSource":
        """Create and return an object that encapsulates the data from a AWS S3 bucket.

        Args:
            uri: The AWS S3 bucket location of the Parquet files as a string.
            access_key: The access key for the S3 bucket. If **None** (default value), then the value
                of environment variable AWS_ACCESS_KEY_ID is used.
            secret_key: The secret key for the S3 bucket. If **None** (default value), then the value
                of environment variable AWS_SECRET_ACCESS_KEY is used.
            tag: The tag name for the data source. If **None** (default value), then the tag **latest**
                will be assigned to the data source.

        Returns:
            An instance of the `DataSource` class. For more information on the methods that are available in
            the returned object, please check the documentation of the `DataSource` class.

        Raises:
            ValueError: If the parameters client and URI are empty or None.
            ValueError: If the input parameters to the API are invalid.
            ConnectionError: If the server address is invalid or not reachable.

        An example function call to the DataSource.s3:

        ```python
            data_source_s3 = DataSource.s3(
                uri="s3://bucket/events.parquet"
            )
        ```
        """
        access_key = (
            access_key if access_key is not None else os.environ["AWS_ACCESS_KEY_ID"]
        )
        secret_key = (
            secret_key
            if secret_key is not None
            else os.environ["AWS_SECRET_ACCESS_KEY"]
        )

        response = Client.post_data(
            relative_url="/data/s3",
            data=dict(uri=uri, access_key=access_key, secret_key=secret_key, tag=tag),
        )

        return DataSource(data_id=response["id"], data_type=response["type"])

    @staticmethod
    def db(
        *,
        host: str,
        database: str,
        table: str,
        port: int = 3306,
        engine: str = "mysql",
        username: Optional[str] = None,
        password: Optional[str] = None,
        tag: Optional[str] = None,
    ) -> "DataSource":
        """Create and return an object that encapsulates the data from a database.

        A static method that creates and returns an object that encapsulates the data
        from a database. In case if access to the database requires authentication,
        the username and password will be read either from the arguments or in the airt
        environment variables.

        The objects created by calling this method won't establish the connection yet.

        Args:
            host: The name of the remote database host machine.
            database: The logical name of the database to establish the connection.
            table: The name of the table in the database.
            port: The port for the database server. If the value is not passed then the
                default port number will be used (e.g. for MySQL, 3306 will be used).
            engine: The name of the database engine. If the value is not passed
                then the default database engine for MySQL will be used.
            username: A valid database user name. If not set (default value "root"),
                it will try to use the value from environment variable AIRT_CLIENT_DB_USERNAME.
            password: The password for the specified user. If not set (default value ""),
                it will try to use the value from environment variable AIRT_CLIENT_DB_PASSWORD.
            tag: The tag name for the data source. If "None" (default value), then the tag **latest**
                will be assigned to the data source.

        Returns:
            An instance of the `DataSource` class. For more information on the methods that
            are available in the returned object, please check the documentation of the
            `DataSource` class.

        Raises:
            ValueError: If the requred parameters are empty or None.
            ValueError: If the requred parameters to the API are invalid.
            ConnectionError: If the server address is invalid or not reachable.

        An example function call to the DataSource.db:

        ```python
        data_source = DataSource.db(
            host="db.staging.airt.ai",
            database="test",
            table="events"
        )
        ```
        """
        username = (
            username
            if username is not None
            else os.environ.get("AIRT_CLIENT_DB_USERNAME", "root")
        )

        password = (
            password
            if password is not None
            else os.environ.get("AIRT_CLIENT_DB_PASSWORD", "")
        )

        _body = dict(
            host=host,
            port=port,
            username=username,
            password=password,
            database=database,
            table=table,
            database_server=engine,
            tag=tag,
        )

        response = Client.post_data(relative_url=f"/data/db", data=_body)

        return DataSource(data_id=response["id"], data_type=response["type"])

    @staticmethod
    def _get_columns() -> list:
        """Returns the list of columns to be added to the DataFrame."""

        return [
            "id",
            "type",
            "tags",
            "created",
            "completed_steps",
            "total_steps",
            "no_of_rows",
            "folder_size",
        ]

    @staticmethod
    def ls(
        offset: int = 0,
        limit: int = 100,
        disabled: bool = False,
        completed: bool = False,
    ) -> pd.DataFrame:
        """Display the list of available datasources.

        Args:
            offset: The number of rows to offset at the beginning of the datasource
                list from the server.If **None**, then the default value **0** will be used.
            limit: The maximum number of rows to return from the server. If **None**,
                then the default value **100** will be used.
            disabled: If set to **True**, then only the deleted datasources will be displayed.
                Else, the default value **False** will be used to display only the list
                of active datasources.
            completed: If set to **True**, then only the datasources that are successfully downloaded
                to airt server will be displayed. Else, the default value **False** will be used to
                display all the datasources including the ones that are created but not yet pulled
                into the airt server.

        Returns:
            A pandas dataframe with the list of available datasources.

        Raises:
            ConnectionError: If the server address is invalid or not reachable.

        An example to list the available datasources:

        ```python
        DataSource.ls()
        ```
        """
        lists = Client.get_data(
            relative_url=f"/data/?disabled={disabled}&completed={completed}&offset={offset}&limit={limit}"
        )

        for _list in lists:
            _list["tags"] = get_tag_str(_list["tags"])

        columns = DataSource._get_columns()

        lists_df = generate_df(lists, columns)

        return add_ready_column(lists_df)

    @staticmethod
    def details(data_id: int) -> pd.DataFrame:
        """Return details of a data source

        Args:
            data_id: The id of the data in the airt service.

        Returns:
            A pandas dataframe with the details of the data source.

        Raises:
            ConnectionError: If the server address is invalid or not reachable.

        An example to get details of a data source (id=1) from airt service:

        ```python
        DataSource.details(data_id=1)
        ```
        """

        details = Client.get_data(relative_url=f"/data/{data_id}")

        details["tags"] = get_tag_str(details["tags"])

        additional_cols = ["user_id", "error", "disabled"]

        columns = DataSource._get_columns() + additional_cols

        details_df = pd.DataFrame(details, index=[0])[columns]

        return add_ready_column(details_df)

    @staticmethod
    def delete(data_id: int) -> pd.DataFrame:
        """Delete a datasource from airt service

        Args:
            data_id: The id of the data in the airt service.

        Returns:
            A pandas DataFrame encapsulating the details of the deleted data id

        Raises:
            ConnectionError: If the server address is invalid or not reachable.

        An example to delete a data source (id=1) from airt service:

        ```python
        DataSource.delete(data_id=1)
        ```
        """

        response = Client.delete_data(relative_url=f"/data/{data_id}")

        response["tags"] = get_tag_str(response["tags"])

        columns = DataSource._get_columns()

        datasource_df = pd.DataFrame(response, index=[0])[columns]

        return add_ready_column(datasource_df)

    @property
    def dtypes(self) -> pd.DataFrame:
        """Return the data type of each column for a data source.

        Returns:
            A pandas dataframe that contains the column names and its data types.

        Raises:
            ValueError: If the input parameters to the API are invalid.
            ConnectionError: If the server address is invalid or not reachable.

        An example to check the dtypes of the connected datasource:

        ```python
        data_source_s3 = DataSource.s3(
            uri="s3://bucket/events.parquet"
        )
        data_source_s3.pull().progress_bar()
        data_source_s3.dtypes
        ```
        """
        dtypes = Client.get_data(relative_url=f"/data/{int(self.id)}/dtypes")
        return pd.DataFrame(dtypes, index=[0])

    @staticmethod
    def tag(data_id: int, name: str) -> pd.DataFrame:
        """Tag an existing datasource in airt service.

        Args:
            data_id: The id of the datasource in airt service for tagging.
            name: The tag name for the datasource.

        Returns:
            A pandas dataframe with the details of the newly tagged datasource.

        Raises:
            ValueError: Incase of invalid data_id.
            ConnectionError: If the server address is invalid or not reachable.

        An example to tag an existing datasource details of a data source (id=1) from airt service:

        ```python
        DataSource.tag(data_id=1, name="v1.0")
        ```
        """
        response = Client.post_data(
            relative_url=f"/data/{data_id}/tag", data=dict(name=name)
        )

        response["tags"] = get_tag_str(response["tags"])

        columns = DataSource._get_columns()

        tag_df = pd.DataFrame(response, index=[0])[columns]

        return add_ready_column(tag_df)

    @staticmethod
    def csv(
        file_path: Union[str, Path],
        index_column: str,
        sort_by: str,
        tag_name: Optional[str] = None,
        deduplicate_data: bool = False,
        blocksize: str = "256MB",
        kwargs_json: Optional[str] = None,
    ) -> "DataSource":
        """Create and return an object that encapsulates the data from a local CSV file.

        Args:
            file_path: The relative or absolute path of the local CSV file as a string or a Path object.
            index_column: The Column name to use as the row labels.
            sort_by: The Column name to sort the data.
            tag_name: The tag name for the datasource.
            deduplicate_data: A boolean flag to handle the duplicate rows in the CSV file. If set to **True** (default value **False**),
                the duplicate rows will be removed from the CSV file before uploading to airt server.
            blocksize: The number of bytes by which to cut up larger files. If None, the default value **256MB** will be used for each file.
            kwargs_json: Any additional parameters for Pandas read_csv method as a JSON string.

        Returns:
            An instance of the `DataSource` class. For more information on the methods that are available in
            the returned object, please check the documentation of the `DataSource` class.

        Raises:
            ValueError: If the csv upload to s3 is unsuccessful.
            ConnectionError: If the server address is invalid or not reachable.

        An example function call to the DataSource.csv:

        ```python
            data_source_csv = DataSource.csv(
                file_path=csv_file_path,
                index_column=index_column_name,
                sort_by=sort_by_column_name
            )
        ```
        """
        # Step 1: get presigned URL
        response = Client.post_data(relative_url=f"/data/csv", data=dict(tag=tag_name))

        # Step 2: download the csv to the s3 bucket
        files = {"file": open(Path(file_path), "rb")}

        s3_response = requests.post(
            response["presigned"]["url"],
            files=files,
            data=response["presigned"]["fields"],
        )

        if not s3_response.status_code == 204:
            raise ValueError(s3_response.text)

        csv_pull_params = dict(
            deduplicate_data=deduplicate_data,
            index_column=index_column,
            sort_by=sort_by,
            blocksize=blocksize,
            kwargs_json=kwargs_json,
        )
        return DataSource(
            data_id=response["id"],
            data_type=response["type"],
            csv_pull_params=csv_pull_params,
        )

    def pull(self) -> "ProgressStatus":
        raise NotImplementedError()

    def head(self) -> pd.DataFrame:
        raise NotImplementedError()

    def train(
        self,
        *,
        client_column: str,
        timestamp_column: Optional[str] = None,
        target_column: str,
        target: str,
        predict_after: timedelta,
    ) -> "airt.client.Model":  # type: ignore
        raise NotImplementedError()

    def push(self, predictions: "airt.client.Prediction"):  # type: ignore
        raise NotImplementedError()


# Cell


@patch
def pull(self: DataSource) -> ProgressStatus:
    """A function to establish the connection with the data source.

    The pull method establishes the connection with the specified `DataSource` and pulls the
    data into the server for further processing. The call to this method is asynchronous and the progress of the connection can be checked
    using the progress bar or the status flag. Please refer to `DataSource` class documentation for more information.

    Returns:
        An instance of `ProgressStatus` class. `ProgressStatus` is a base class for querying status of a remote operation. For more information
        please refer to `ProgressStatus` class documentation.

    Raises:
        ValueError: If the input parameters to the API are invalid.
        ConnectionError: If the server address is invalid or not reachable.

    Below example shows establishing a connection with the s3 bucket:

    ```python

    Client.get_token()
    data_source_s3 = DataSource.s3(
        uri="s3://test-airt-service/ecommerce_behavior"
    )

    data_source_s3.pull().progress_bar()
    ```
    """
    if self.data_type == "csv":
        Client.post_data(
            relative_url=f"/data/{int(self.id)}/csv/pull", data=self.csv_pull_params
        )
    else:
        Client.get_data(relative_url=f"/data/{int(self.id)}/pull")

    return ProgressStatus(relative_url=f"/data/{int(self.id)}")


# Cell


@patch
def head(self: DataSource) -> pd.DataFrame:
    """A function to display the first few records of the data source.

    After successfully pulling the data into the server, the head function can be used
    to display the first few records of the downloaded data.

    Returns:
        A pandas dataframe that displays the first few records of the connected data source.

    Raises:
        ValueError: If the input parameters to the API are invalid.
        ConnectionError: If the server address is invalid or not reachable.

    An example to show the first few records of the data source:

    ```python

    Client.get_token()
    data_source_s3 = DataSource.s3(
        uri="s3://test-airt-service/ecommerce_behavior"
    )
    data_source_s3.pull().progress_bar()

    data_source_s3.head()
    ```
    """
    response = Client.get_data(relative_url=f"/data/{int(self.id)}/head")
    return pd.DataFrame(response)


# Cell


@patch
def train(
    self: DataSource,
    *,
    client_column: str,
    timestamp_column: Optional[str] = None,
    target_column: str,
    target: str,
    predict_after: timedelta,
) -> Model:
    """A method to train the ML model on the connected `DataSource`.

    This method trains the model for predicting which clients are most likely to have a specified
    event in the future. The call to this method is asynchronous and the progress of the connection
    can be checked using the progress bar method or the status flag attribute available in the `DataSource` class.
    For more information on the model, please check the documentation of `Model` class.

    Args:
        client_column: The name of the column that uniquely identifies the users/clients as string.
        timestamp_column: Name of the timestamp_column specifying the time of an
            occurred event as a string. If the value is not passed then the None will be
        target_column: Name of the target column that captures the type of event as string. This will
            be used for training the model as well as for making predictions for our target event.
        target: Name of the target event for which the model needs to be trained to make predictions.
            You can pass regular expressions as well to this parameter for making predictions for more than one event.
            For example, the passing "*checkout will train a model to predict which users will do any kind of a
            checkout event.
        predict_after: Time delta in hours of the expected target event mentioned as timedelta.

    Returns:
        An instance of the `Model` class.

    Raises:
        ValueError: If any of the required parameters are empty or None.
        ValueError: If the input parameters to the API are invalid.
        ConnectionError: If the server address is invalid or not reachable.

    Below is an example for training a model to predict which users will perform a purchase event ("*purchase") 3 hours before they acctually do it:

    ```python
    from datetime import timedelta

    model = data_source_s3.train(
        client_column="user_id",
        target_column="event_type",
        target="*purchase",
        predict_after=timedelta(hours=3)
    )

    model.progress_bar()
    ```
    """
    response = Client.post_data(
        relative_url=f"/model/train",
        data=dict(
            data_id=int(self.id),
            client_column=client_column,
            target_column=target_column,
            target=target,
            predict_after=int(predict_after.total_seconds()),
        ),
    )

    return Model(model_id=response["id"])


# Cell


@patch
def push(self: DataSource, predictions: Prediction):
    """A function to push the prediction results into the target data source.

    For more information on the supported datasources, please refer to the documentation on `DataSource` class.

    Args:
        predictions: An instance of the `Prediction` class.

    Raises:
        ValueError: If the input parameters to the API are invalid.
        ConnectionError: If the server address is invalid or not reachable.

    The below example illustrates pushing the prediction results to a database:

    ```python
    from datetime import timedelta

    Client.get_token()

    data_source_s3 = DataSource.s3(
        uri="s3://test-airt-service/ecommerce_behavior"
    )
    data_source_s3.pull().progress_bar()

    model = data_source_s3.train(
        client_column="user_id",
        target_column="event_type",
        target="*purchase",
        predict_after=timedelta(hours=3),
    )

    predictions = model.predict()

    data_source_pred = DataSource.s3(
        uri="s3://target-bucket"
    )

    progress = data_source_pred.push(predictions)
    progress.progress_bar()
    ```
    """
    return predictions.push(data_source=self)



# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/CLI_Token.ipynb (unless otherwise specified).

__all__ = ["logger"]

# Cell

from typing import *

# Internal Cell

import os
import typer
import logging

from ..client import Client
from ..logger import get_logger, set_level

# Internal Cell

app = typer.Typer()

# Cell

logger = get_logger(__name__)

# Internal Cell


def token(
    username: Optional[str] = typer.Option(
        None,
        "--username",
        "-u",
        help="Username for your developer account. If not set (default value None), "
        "it will try to use the value from environment variable AIRT_SERVICE_USERNAME.",
    ),
    password: Optional[str] = typer.Option(
        None,
        "--password",
        "-p",
        help="Password for your developer account. If not set (default value None), "
        "it will try to use the value from environment variable AIRT_SERVICE_PASSWORD.",
    ),
    server: Optional[str] = typer.Option(
        None,
        "--server",
        "-s",
        help="Server address used to connect to. If not set (default value None), it will "
        "try to use the value from environment variable AIRT_SERVER_URL. If the variable is not "
        "set as well, then the default public server will be used. You should leave this to default "
        "value unless you are running your own server (please contact us for that possibility by email info@airt.ai).",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output authentication token only.",
    ),
    debug: bool = typer.Option(
        False, "--debug", "-d", help="Set logger level to DEBUG and output everything."
    ),
) -> None:
    """Get application token for airt service from a username/password pair.

    Before you can use the service, you must acquire a username and password for your developer account.
    Please contact us by email info@airt.ai to get one.

    The username, password, and server address can either be passed explicitly while calling this command
    or can be stored in environment variables AIRT_SERVICE_USERNAME, AIRT_SERVICE_PASSWORD,
    and AIRT_SERVER_URL.

    Upon successful authentication, the airt services will be available to access.
    """
    try:
        if debug:
            set_level(logging.DEBUG)
        else:
            set_level(logging.WARNING)

        Client.get_token(username=username, password=password, server=server)

        if quiet:
            typer.echo(Client.auth_token)
        else:
            typer.echo(f"token: {Client.auth_token}")

    except KeyError as e:
        typer.echo(message=f"Error: {e}", err=True)
        typer.echo(f"\nTry 'airt token --help' for help.")
        raise typer.Exit(code=1)

    except Exception as e:
        typer.echo(message=f"Error: {e}", err=True)
        raise typer.Exit(code=1)


# Internal Cell


app.command()(token)



# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/CLI_Pred.ipynb (unless otherwise specified).

__all__ = ["logger"]

# Cell

from typing import *

# Internal Cell

import os
import typer
from typer import echo
from tabulate import tabulate

from . import helper
from ..logger import get_logger, set_level

# Internal Cell

app = typer.Typer()

# Cell

logger = get_logger(__name__)

# Internal Cell


@app.command()
@helper.requires_auth_token
def ls(
    offset: int = typer.Option(
        0,
        "--offset",
        "-o",
        help="The number of records to offset at the beginning of the predictions list. If None, then the default value 0 will be used.",
    ),
    limit: int = typer.Option(
        100,
        "--limit",
        "-l",
        help="The maximum number of records to return from the server. If None, then the default value 100 will be used.",
    ),
    disabled: bool = typer.Option(
        False,
        "--disabled",
        help="If set to True, then only the deleted predictions will be displayed. Else, the default value "
        "False will be used to display only the list of active predictions.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output only ids of prediction separated by space.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> None:
    """List available predictions."""

    from ..client import Prediction

    df = Prediction.ls(offset=offset, limit=limit, disabled=disabled)

    df["created"] = helper.humanize_date(df["created"])

    if quiet:
        ids = df["id"].astype(str).to_list()
        typer.echo("\n".join(ids))
    else:
        typer.echo(tabulate(df, headers="keys", tablefmt="plain", showindex=False))


# Internal Cell


@app.command()
@helper.requires_auth_token
def details(
    id: int = typer.Argument(
        ...,
        help="The prediction ID in the airt service.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> None:
    """Return metadata of the given prediction."""

    from ..client import Prediction

    df = Prediction.details(id=int(id))

    df["created"] = helper.humanize_date(df["created"])

    typer.echo(tabulate(df, headers="keys", tablefmt="plain", showindex=False))


# Internal Cell


@app.command()
@helper.requires_auth_token
def rm(
    id: int = typer.Argument(
        ...,
        help="The prediction ID in the airt service.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output the deleted prediction id only.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> None:
    """Remove the prediction from airt service."""

    from ..client import Prediction

    df = Prediction.delete(id=int(id))

    df["created"] = helper.humanize_date(df["created"])

    if quiet:
        typer.echo(df.iloc[0]["id"])
    else:
        typer.echo(tabulate(df, headers="keys", tablefmt="plain", showindex=False))


# Internal Cell


@app.command("to-pandas")
@helper.requires_auth_token
def to_pandas(
    id: int = typer.Argument(
        ...,
        help="The prediction ID in the airt service.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> None:
    """Convert the predicted results into a Pandas DataFrame object."""

    from ..client import Prediction

    pred = Prediction(prediction_id=int(id))

    df = pred.to_pandas()

    typer.echo(tabulate(df, headers="keys", tablefmt="plain", showindex=False))


# Internal Cell


@app.command()
@helper.requires_auth_token
def push(
    data_id: int = typer.Option(
        ...,
        "--data",
        help="The data ID in the airt service.",
    ),
    pred_id: int = typer.Option(
        ...,
        "--pred",
        help="The prediction ID in the airt service.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output the pushed prediction id only.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> None:
    """Push the prediction results into the target data source."""

    from ..client import DataSource, Prediction

    datasource = DataSource(int(data_id))
    prediction = Prediction(prediction_id=int(pred_id), datasource_id=int(data_id))

    progress = prediction.push(datasource)

    if quiet:
        progress.wait()

        typer.echo(prediction.prediction_id)
    else:
        typer.echo(
            f"Pushing the results of prediction id: {prediction.prediction_id} into the datasource id: {prediction.datasource_id}"
        )

        progress.progress_bar()



# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/API_ProgressStatus.ipynb (unless otherwise specified).

__all__ = ["ProgressStatus"]

# Cell

from typing import *

# Internal Cell

from time import sleep
from datetime import datetime, timedelta

from tqdm import tqdm

from fastcore.foundation import patch

from .client import Client
from ..logger import get_logger, set_level

# Internal Cell

logger = get_logger(__name__)

# Cell


class ProgressStatus:
    """A base class for querying status of a remote operation"""

    def __init__(self, relative_url: str):
        """Constructs a new ProgressStatus instance.

        Warning:
            Do not construct this object directly by calling the constructor, please use `DataSource.pull` or `DataSource.train` or `Model.predict` functions instead.

        Args:
            relative_url: The relative URL to the rest API endpoint.
        """
        self.relative_url = relative_url

    def is_ready(self) -> bool:
        """A function to check if the method's progress is completed.

        Returns:
            True if the progress if completed, else False.
        """
        response = Client.get_data(relative_url=self.relative_url)
        return response["completed_steps"] == response["total_steps"]

    def progress_bar(self, sleep_for: Union[int, float] = 5, timeout: int = 0):
        """Blocks execution while waiting for remote action to be completed and displays a progress bar indicating the completion status.

        Args:
            sleep_for: The time interval in seconds between successive API calls to ping the server for fetching the completed steps.
            timeout: The maximum time allowed in seconds for the asynchronous call to complete the process. If the timeout
                exceeds and the process is yet to complete, then the progress_bar will be terminated.

        Raises:
            TimeoutError: in case of timeout

        """
        total_steps = Client.get_data(relative_url=self.relative_url)["total_steps"]
        with tqdm(total=total_steps) as pbar:
            started_at = datetime.now()
            while True:
                if (0 < timeout) and (datetime.now() - started_at) > timedelta(
                    seconds=timeout
                ):
                    raise TimeoutError()

                response = Client.get_data(relative_url=self.relative_url)
                completed_steps = response["completed_steps"]

                pbar.update(completed_steps)

                if completed_steps == total_steps:
                    break

                sleep(sleep_for)

    def wait(self, sleep_for: Union[int, float] = 1, timeout: int = 0):
        raise NotImplementedError()


# Cell


@patch
def wait(self: ProgressStatus, sleep_for: Union[int, float] = 1, timeout: int = 0):
    """Blocks execution while waiting for remote action to be completed.

    Args:
        sleep_for: The time interval in seconds between successive API calls to ping the server for fetching the completed steps.
        timeout: The maximum time allowed in seconds for the asynchronous call to complete the process. If the timeout
            exceeds and the process is yet to complete, then the progress_bar will be terminated.

    Raises:
        TimeoutError: in case of timeout
    """
    started_at = datetime.now()
    while True:
        if (0 < timeout) and (datetime.now() - started_at) > timedelta(seconds=timeout):
            raise TimeoutError()

        if self.is_ready():
            return

        sleep(sleep_for)



# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/CLI_Helper.ipynb (unless otherwise specified).

__all__ = [
    "logger",
    "requires_auth_token",
    "humanize_date",
    "humanize_number",
    "humanize_size",
]

# Cell

import logging
from typing import *

# Internal Cell

import os
from contextlib import contextmanager
import functools

import typer
import humanize
import datetime as dt
import pandas as pd

from ..client import Client
from ..logger import get_logger, set_level

# Cell

logger = get_logger(__name__)

# Internal Cell


@contextmanager
def authenicate_user():
    Client(
        auth_token=os.environ["AIRT_SERVICE_TOKEN"],
        server=os.environ.get("AIRT_SERVER_URL", None),
    )

    yield


# Cell


def requires_auth_token(func):
    @functools.wraps(func)
    def wrapper_decorator(*args, **kwargs):
        try:

            if ("debug" in kwargs) and kwargs["debug"]:
                set_level(logging.DEBUG)
            else:
                set_level(logging.WARNING)

            with authenicate_user():
                # Do something before
                value = func(*args, **kwargs)
                # Do something after

        except KeyError as e:
            typer.echo(
                message=f"KeyError: The environment variable {e} is not set.", err=True
            )

            if "'AIRT_SERVICE_TOKEN'" in str(e):
                typer.echo(
                    f"\nPlease run the command 'airt token' to get the application token and set it in the "
                    f"environment variable `AIRT_SERVICE_TOKEN`."
                )
                typer.echo(f"\nTry 'airt token --help' for help.")

            raise typer.Exit(code=1)

        except Exception as e:
            typer.echo(message=f"Error: {e}", err=True)
            raise typer.Exit(code=1)
        return value

    return wrapper_decorator


# Cell


def humanize_date(s: pd.Series) -> pd.Series:
    return s.apply(
        lambda date: humanize.naturaltime(
            dt.datetime.now() - dt.datetime.strptime(date, "%Y-%m-%dT%H:%M:%S")
        )
    )


# Cell


def humanize_number(s: pd.Series) -> pd.Series:
    return s.apply(
        lambda num: humanize.intcomma(int(num)) if pd.notna(num) else "unknown"
    )


# Cell


def humanize_size(s: pd.Series) -> pd.Series:
    return s.apply(
        lambda size: humanize.naturalsize(size) if pd.notna(size) else "unknown"
    )



# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/API_Keys.ipynb (unless otherwise specified).

__all__ = ["APIKey"]

# Cell

from typing import *

# Internal Cell

import os

from datetime import datetime, timedelta, date

import pandas as pd
from fastcore.foundation import patch

from .client import Client
from ..logger import get_logger, set_level
from ..helper import get_data, post_data, delete_data, generate_df

# Internal Cell

logger = get_logger(__name__)

# Cell


class APIKey:
    """A class for managing the API keys in the airt service.

    Both the API key and the token can be used for accessing the airt services. However, there is a slight difference in generating and managing the two.

    For generating the API key, you first need to get the token. Please refer to the documentation of the `Client.get_token` method for generating one.

    Once successfully authenticated, you can create any number of API keys and set an expiration date for each key individually. Also, you can use the other
    methods available in the APIKey class to list, revoke the keys at any time.
    """

    @staticmethod
    def create(name: str, expiry: Union[int, timedelta, datetime]) -> Dict[str, str]:
        """Create a new API key in airt service

        Args:
            name: Name for the new API key.
            expiry: The validity for the newly created API key. The expiry can be an integer representing the number of days till expiry or
                can be an instance of timedelta (timedelta(days=x)) or can be an instance of datetime.

        Returns:
            The access token and the type of the token as a dictionary.

        Raises:
            ValueException: If the user is not authenticated.
            ConnectionError: If the server address is invalid or not reachable.

        A simple example to create a new API key with 1 year validity

        ```python
        Client.get_token()

        APIKey.create(
            name="sample_key"
            expiry=365
        )
        ```
        """

        if isinstance(expiry, int):
            delta = datetime.now() + timedelta(days=expiry)
        elif isinstance(expiry, timedelta):
            delta = datetime.now() + expiry
        else:
            delta = expiry

        expiry_date = delta.strftime("%Y-%m-%dT%H:%M")

        return Client.post_data(
            relative_url="/apikey",
            data=dict(name=name, expiry=expiry_date),
        )

    @staticmethod
    def _get_columns() -> list:
        """Returns the list of columns to be added to the DataFrame."""

        return ["id", "name", "created", "expiry", "disabled"]

    @staticmethod
    def ls(
        offset: int = 0,
        limit: int = 100,
        include_disabled: bool = False,
    ) -> pd.DataFrame:
        """Display the list of available API keys.

        Args:
            offset: The number of rows to offset at the beginning of the API keys
                list from the server.If **None**, then the default value **0** will be used.
            limit: The maximum number of rows to return from the server. If **None**,
                then the default value **100** will be used.
            include_disabled: If set to **True**, then the disabled API keys will also be
                included in the result.

        Returns:
            A pandas dataframe with the list of available API keys.

        Raises:
            ConnectionError: If the server address is invalid or not reachable.

        An example to list the available datasources:

        ```python
        APIKey.ls()
        ```
        """

        lists = Client.get_data(
            relative_url=f"/apikey/?include_disabled={include_disabled}&offset={offset}&limit={limit}"
        )

        columns = APIKey._get_columns()

        return generate_df(lists, columns)

    @staticmethod
    def details(api_key_id: int) -> pd.DataFrame:
        """Return details of an API key

        Args:
            api_key_id: The id of the API key in the airt service.

        Returns:
            A pandas dataframe with the details of the API key.

        Raises:
            ValueException: If the api_key_id is invalid.
            ConnectionError: If the server address is invalid or not reachable.

        An example to get details of an API key (id=1) from airt service:

        ```python
        APIKey.details(api_key_id=1)
        ```
        """

        details = Client.get_data(relative_url=f"/apikey/{api_key_id}")

        columns = APIKey._get_columns()

        return pd.DataFrame(details, index=[0])[columns]

    @staticmethod
    def delete(api_key_id: int) -> pd.DataFrame:
        """Delete an API key from airt service

        Args:
            api_key_id:  The id of the API key in the airt service.

        Returns:
            A pandas dataframe with the details of the deleted API key.

        Raises:
            ValueException: If the api_key_id is invalid.
            ConnectionError: If the server address is invalid or not reachable.

        An example to delete an API key (id=1) from airt service:

        ```python
        APIKey.delete(api_key_id=1)
        ```
        """

        response = Client.delete_data(relative_url=f"/apikey/{api_key_id}")

        columns = APIKey._get_columns()

        return pd.DataFrame(response, index=[0])[columns]



# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/CLI_DataSource.ipynb (unless otherwise specified).

__all__ = ["logger"]

# Cell

from typing import *

# Internal Cell

import os

import typer
from typer import echo
from tabulate import tabulate
import datetime as dt
import pandas as pd

from ..client import Client
from . import helper
from ..logger import get_logger, set_level

# Internal Cell

app = typer.Typer()

local_ds_app = typer.Typer()

app.add_typer(
    local_ds_app,
    name="local",
    short_help="Pull the data source from your local machine to airt server.",
    help="Pull the data source from your local machine to airt server. "
    "Currently, we support reading and pushing the data from a CSV file. "
    "We plan to support more data source formats in the future.",
)

# Cell

logger = get_logger(__name__)

# Internal Cell


@app.command()
@helper.requires_auth_token
def s3(
    uri: str = typer.Argument(
        ..., help="The AWS S3 bucket location of the Parquet files as a string."
    ),
    access_key: Optional[str] = typer.Option(
        None,
        help="The access key for the S3 bucket. If None (default value), then the value of environment variable AWS_ACCESS_KEY_ID is used.",
    ),
    secret_key: Optional[str] = typer.Option(
        None,
        help="The secret key for the S3 bucket. If None (default value), then the value of environment variable AWS_SECRET_ACCESS_KEY is used.",
    ),
    tag: Optional[str] = typer.Option(
        None,
        help="The tag name for the data source. If None (default value), then the tag latest will be assigned to the data source.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output data id only.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
):
    """Pull the data source from a S3 bucket to airt server."""

    from ..client import DataSource

    ds = DataSource.s3(uri=uri, access_key=access_key, secret_key=secret_key, tag=tag)

    progress = ds.pull()

    if quiet:
        progress.wait()

        typer.echo(f"{ds.id}")
    else:
        typer.echo(f"Pulling data ID: {ds.id}")

        progress.progress_bar()


# Internal Cell


@app.command()
@helper.requires_auth_token
def db(
    host: str = typer.Option(..., help="The name of the remote database host machine."),
    database: str = typer.Option(
        ..., help="The logical name of the database to establish the connection."
    ),
    table: str = typer.Option(..., help="The name of the table in the database."),
    port: int = typer.Option(
        3306,
        help="The port for the database server. If the value is not passed then the default port number will be used (e.g. for MySQL, 3306 will be used)",
    ),
    engine: str = typer.Option(
        "mysql",
        help="""The name of the database engine. If the value is not passed then the
            default database engine for MySQL will be used.""",
    ),
    username: Optional[str] = typer.Option(
        None,
        "--username",
        "-u",
        help="A valid database user name. If not set (default value root), it will try to use the value from environment variable AIRT_CLIENT_DB_USERNAME.",
    ),
    password: Optional[str] = typer.Option(
        None,
        "--password",
        "-p",
        help="The password for the specified user. If not set (default value ), it will try to use the value from environment variable AIRT_CLIENT_DB_PASSWORD.",
    ),
    tag: Optional[str] = typer.Option(
        None,
        help="The tag name for the data source. If None (default value), then the tag latest will be assigned to the data source.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output data id only.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
):
    """Creates an object that encapsulates the data from a database."""

    from ..client import DataSource

    ds = DataSource.db(
        host=host,
        database=database,
        port=port,
        table=table,
        engine=engine,
        username=username,
        password=password,
        tag=tag,
    )

    progress = ds.pull()

    if quiet:
        progress.wait()
        typer.echo(f"{ds.id}")
    else:
        typer.echo(f"Pulling data ID: {ds.id}")
        progress.progress_bar()


# Internal Cell


@app.command()
@helper.requires_auth_token
def ls(
    offset: int = typer.Option(
        0,
        "--offset",
        "-o",
        help="The number of records to offset at the beginning of the datasource list. If None, then the default value 0 will be used.",
    ),
    limit: int = typer.Option(
        100,
        "--limit",
        "-l",
        help="The maximum number of records to return from the server. If None, then the default value 100 will be used.",
    ),
    disabled: bool = typer.Option(
        False,
        "--disabled",
        help="If set to True, then only the deleted datasources will be displayed. Else, the default value "
        "False will be used to display only the list of active datasources.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output only ids of datasource separated by space",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> None:
    """List available data sources."""

    from ..client import DataSource

    df = DataSource.ls(offset=offset, limit=limit, disabled=disabled)

    df["created"] = helper.humanize_date(df["created"])
    df["no_of_rows"] = helper.humanize_number(df["no_of_rows"])
    df["folder_size"] = helper.humanize_size(df["folder_size"])

    if quiet:
        ids = df["id"].astype(str).to_list()
        typer.echo("\n".join(ids))
    else:
        typer.echo(tabulate(df, headers="keys", tablefmt="plain", showindex=False))


# Internal Cell


@app.command()
@helper.requires_auth_token
def dtypes(
    id: str = typer.Argument(
        ...,
        help="The data ID in the airt service.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> None:
    """Return the dtype of each column for the given data source."""

    from ..client import DataSource

    ds = DataSource(data_id=int(id))
    dtypes = ds.dtypes.T.rename(columns={0: "dtype"})
    typer.echo(dtypes)


# Internal Cell


@app.command()
@helper.requires_auth_token
def head(
    id: str = typer.Argument(
        ...,
        help="The data ID in the airt service.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> None:
    """Return first few records for the given data ID."""

    from ..client import DataSource

    ds = DataSource(data_id=int(id))
    typer.echo(tabulate(ds.head(), headers="keys", tablefmt="plain", showindex=False))


# Internal Cell


@app.command()
@helper.requires_auth_token
def train(
    id: str = typer.Option(
        ...,
        "--data_id",
        "-id",
        help="The data ID in the airt service.",
    ),
    client_column: str = typer.Option(
        ...,
        "--client_column",
        help="The name of the column that uniquely identifies the users/clients as string.",
    ),
    timestamp_column: Optional[str] = typer.Option(
        None,
        "--timestamp_column",
        help="Name of the timestamp_column specifying the time of an occurred event as a string.",
    ),
    target_column: str = typer.Option(
        ...,
        "--target_column",
        help="Name of the target event for which the model needs to be trained to make predictions.",
    ),
    target: str = typer.Option(
        ...,
        "--target",
        help="Name of the target event for which the model needs to be trained to make predictions.",
    ),
    predict_after: str = typer.Option(
        ...,
        "--predict_after",
        help="Time delta in hours of the expected target event.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output model id only.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
):
    """Train a ML model for the given data source."""

    from ..client import DataSource
    from datetime import timedelta

    ds = DataSource(data_id=int(id))

    model = ds.train(
        client_column=client_column,
        target_column=target_column,
        target=target,
        predict_after=timedelta(hours=int(predict_after)),
        timestamp_column=timestamp_column,
    )

    if quiet:
        model.wait()
        typer.echo(f"{model.model_id}")
    else:
        typer.echo(f"Training started for Model ID: {model.model_id}")
        model.progress_bar()


# Internal Cell


@app.command()
@helper.requires_auth_token
def details(
    id: int = typer.Argument(
        ...,
        help="The data ID in the airt service.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> None:
    """Return metadata of the given data source."""

    from ..client import DataSource

    df = DataSource.details(data_id=int(id))

    df["created"] = helper.humanize_date(df["created"])
    df["no_of_rows"] = helper.humanize_number(df["no_of_rows"])
    df["folder_size"] = helper.humanize_size(df["folder_size"])

    typer.echo(tabulate(df, headers="keys", tablefmt="plain", showindex=False))


# Internal Cell


@app.command()
@helper.requires_auth_token
def rm(
    id: int = typer.Argument(
        ...,
        help="The data ID in the airt service.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output the deleted data id only.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> None:
    """Remove the data source from airt service."""

    from ..client import DataSource

    df = DataSource.delete(data_id=int(id))

    df["created"] = helper.humanize_date(df["created"])
    df["no_of_rows"] = helper.humanize_number(df["no_of_rows"])
    df["folder_size"] = helper.humanize_size(df["folder_size"])

    if quiet:
        typer.echo(df.iloc[0]["id"])
    else:
        typer.echo(tabulate(df, headers="keys", tablefmt="plain", showindex=False))


# Internal Cell


@app.command()
@helper.requires_auth_token
def push(
    data_id: int = typer.Option(
        ...,
        "--data",
        help="The data ID in the airt service.",
    ),
    pred_id: int = typer.Option(
        ...,
        "--pred",
        help="The prediction ID in the airt service.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output the pushed prediction id only.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> None:
    """Push the prediction results into the target data source."""

    from ..client import DataSource, Prediction

    datasource = DataSource(int(data_id))
    prediction = Prediction(prediction_id=int(pred_id), datasource_id=int(data_id))

    progress = datasource.push(prediction)

    if quiet:
        progress.wait()

        typer.echo(prediction.prediction_id)
    else:
        typer.echo(
            f"Pushing the results of prediction id: {prediction.prediction_id} into the datasource id: {prediction.datasource_id}"
        )

        progress.progress_bar()


# Internal Cell


@app.command()
@helper.requires_auth_token
def tag(
    id: int = typer.Option(
        ...,
        "--data_id",
        "-id",
        help="The data ID in the airt service.",
    ),
    name: str = typer.Option(
        ...,
        "--tag_name",
        "-n",
        help="The tag name for the datasource.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> None:
    """Tag an existing datasource in airt service."""

    from ..client import DataSource

    df = DataSource.tag(data_id=int(id), name=name)

    df["created"] = helper.humanize_date(df["created"])
    df["no_of_rows"] = helper.humanize_number(df["no_of_rows"])
    df["folder_size"] = helper.humanize_size(df["folder_size"])

    typer.echo(tabulate(df, headers="keys", tablefmt="plain", showindex=False))


# Internal Cell


@local_ds_app.command()
@helper.requires_auth_token
def csv(
    file_path: str = typer.Option(
        ...,
        "--file_path",
        "-f",
        help="The relative or absolute path of the csv file.",
    ),
    index_column: str = typer.Option(
        ...,
        "--index_column",
        "-i",
        help="The Column name to use as the row labels.",
    ),
    sort_by: str = typer.Option(
        ...,
        "--sort_by",
        "-s",
        help="The Column name to sort the data.",
    ),
    tag_name: Optional[str] = typer.Option(
        None,
        "--tag_name",
        "-t",
        help="The tag name for the datasource.",
    ),
    deduplicate_data: bool = typer.Option(
        False,
        "--deduplicate_data",
        "-dd",
        help="A boolean flag to handle the duplicate rows in the CSV file. If set to **True** (default value **False**), "
        "the duplicate rows will be removed from the CSV file before uploading to airt server.",
    ),
    blocksize: str = typer.Option(
        "256MB",
        "--blocksize",
        "-bs",
        help="The number of bytes by which to cut up larger files. If None, the default value **256MB** will be used for each file.",
    ),
    kwargs_json: Optional[str] = typer.Option(
        None,
        "--kwargs_json",
        "-json",
        help="Any additional parameters for Pandas read_csv method as a JSON string.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output data id only.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> None:
    """Pull the data source from a local CSV file to airt server."""

    from ..client import DataSource

    ds = DataSource.csv(
        file_path=file_path,
        index_column=index_column,
        sort_by=sort_by,
        tag_name=tag_name,
        deduplicate_data=deduplicate_data,
        blocksize=blocksize,
        kwargs_json=kwargs_json,
    )

    progress = ds.pull()

    if quiet:
        progress.wait()

        typer.echo(f"{ds.id}")
    else:
        typer.echo(f"Pulling data ID: {ds.id}")

        progress.progress_bar()



# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {
    "logger": "CLI_Version.ipynb",
    "Client": "Client.ipynb",
    "DataSource": "Client.ipynb",
    "DataSource.pull": "API_DataSource.ipynb",
    "DataSource.head": "API_DataSource.ipynb",
    "DataSource.train": "API_DataSource.ipynb",
    "DataSource.push": "API_DataSource.ipynb",
    "ensure_is_instance": "API_Helper.ipynb",
    "get_base_url": "API_Helper.ipynb",
    "post_data": "API_Helper.ipynb",
    "get_data": "API_Helper.ipynb",
    "delete_data": "API_Helper.ipynb",
    "add_ready_column": "API_Helper.ipynb",
    "generate_df": "API_Helper.ipynb",
    "get_tag_str": "API_Helper.ipynb",
    "APIKey": "Client.ipynb",
    "Model": "Client.ipynb",
    "Model.evaluate": "API_Model.ipynb",
    "Model.predict": "API_Model.ipynb",
    "Prediction": "Client.ipynb",
    "Prediction.to_pandas": "API_Prediction.ipynb",
    "Prediction.push": "API_Prediction.ipynb",
    "ProgressStatus": "Client.ipynb",
    "ProgressStatus.wait": "API_ProgressStatus.ipynb",
    "User": "Client.ipynb",
    "activated": "ActivateTestingEnvironment.ipynb",
    "app": "CLI_Version.ipynb",
    "local_ds_app": "CLI_DataSource.ipynb",
    "s3": "CLI_DataSource.ipynb",
    "db": "CLI_DataSource.ipynb",
    "ls": "CLI_User.ipynb",
    "dtypes": "CLI_DataSource.ipynb",
    "head": "CLI_DataSource.ipynb",
    "train": "CLI_DataSource.ipynb",
    "details": "CLI_Pred.ipynb",
    "rm": "CLI_Pred.ipynb",
    "push": "CLI_Pred.ipynb",
    "tag": "CLI_DataSource.ipynb",
    "csv": "CLI_DataSource.ipynb",
    "authenicate_user": "CLI_Helper.ipynb",
    "requires_auth_token": "CLI_Helper.ipynb",
    "humanize_date": "CLI_Helper.ipynb",
    "humanize_number": "CLI_Helper.ipynb",
    "humanize_size": "CLI_Helper.ipynb",
    "create": "CLI_User.ipynb",
    "predict": "CLI_Model.ipynb",
    "evaluate": "CLI_Model.ipynb",
    "to_pandas": "CLI_Pred.ipynb",
    "token": "CLI_Token.ipynb",
    "disable": "CLI_User.ipynb",
    "enable": "CLI_User.ipynb",
    "update": "CLI_User.ipynb",
    "version": "CLI_Version.ipynb",
    "logging_level": "Logger.ipynb",
    "CLILoggerHandler": "Logger.ipynb",
    "handler": "Logger.ipynb",
    "supress_timestamps": "Logger.ipynb",
    "get_logger": "Logger.ipynb",
    "should_supress_timestamps": "Logger.ipynb",
    "set_level": "Logger.ipynb",
}

modules = [
    "components/client.py",
    "components/datasource.py",
    "helper.py",
    "components/api_key.py",
    "components/model.py",
    "components/prediction.py",
    "components/progress_status.py",
    "components/user.py",
    "testing/activate_by_import.py",
    "cli/cli.py",
    "cli/ds.py",
    "cli/helper.py",
    "cli/api_key.py",
    "cli/model.py",
    "cli/pred.py",
    "cli/token.py",
    "cli/user.py",
    "cli/version.py",
    "client.py",
    "logger.py",
]

doc_url = "//"

git_url = "https://gitlab.com/airt.ai/airt-client/tree/main/"


def custom_doc_links(name):
    return None



# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/API_Prediction.ipynb (unless otherwise specified).

__all__ = ["Prediction"]

# Cell

from typing import *

# Internal Cell

import os

import pandas as pd
from fastcore.foundation import patch

from ..logger import get_logger, set_level
from ..helper import get_data, post_data, delete_data, add_ready_column, generate_df

from .client import Client
from .progress_status import ProgressStatus

# Internal Cell

logger = get_logger(__name__)

# Cell


class Prediction(ProgressStatus):
    """A class to run predictions on the data source.

    The `Prediction` class is automatically instantiated by calling the `DataSource.train` method of a `DataSource` instance.
    Currently, it is the only way to instantiate this class. The returned object will have utility methods like converting the prediction
    results into a pandas dataframe and pushing the prediction results into one of the supported data sources etc.,

    For more information on the supported data sources, please refer to the documentation on `DataSource` class
    """

    def __init__(self, prediction_id: int, datasource_id: Optional[int] = None):
        """Constructs a new `Prediction` instance

        Warning:
            Do not construct this object directly by calling the constructor, instead please use
            `Model.predict` method of the Model instance.

        Args:
            prediction_id: ID of the prediction in the airt service
            datasource_id: The data ID used for running the prediction
        """
        self.prediction_id = prediction_id
        self.datasource_id = datasource_id
        ProgressStatus.__init__(self, relative_url=f"/prediction/{self.prediction_id}")

    @staticmethod
    def _get_columns(col_type: str = "ls_del_cols") -> list:
        """Returns the list of columns to be added to the DataFrame."""

        return ["id", "created", "total_steps", "completed_steps"]

    @staticmethod
    def ls(
        offset: int = 0,
        limit: int = 100,
        disabled: bool = False,
        completed: bool = False,
    ) -> pd.DataFrame:
        """Display the list of available predictions in airt service.

        Args:
            offset: The number of rows to offset at the beginning of the predictions
                list from the server.If **None**, then the default value **0** will be used.
            limit: The maximum number of rows to return from the server. If **None**,
                then the default value **100** will be used.
            disabled: If set to **True**, then only the deleted predictions will be displayed.
                Else, the default value **False** will be used to display only the list
                of active predictions.
            completed: If set to **True**, then only the predictions that are successfully completed
                in airt server will be displayed. Else, the default value **False** will be used
                to display all the predictions including the ones that are yet to finish the prediction.

        Returns:
            A pandas dataframe with the list of available predictions.

        Raises:
            ConnectionError: If the server address is invalid or not reachable.

        An example to list the available predictions:

        ```python
        Prediction.ls()
        ```
        """
        predictions = Client.get_data(
            relative_url=f"/prediction/?disabled={disabled}&completed={completed}&offset={offset}&limit={limit}"
        )

        columns = Prediction._get_columns()

        predictions_df = generate_df(predictions, columns)

        return add_ready_column(predictions_df)

    @staticmethod
    def details(id: int) -> pd.DataFrame:
        """Return details of a prediction

        Args:
            id: The id of the prediction in the airt service.

        Returns:
            A pandas dataframe with the details of the prediction.

        Raises:
            ConnectionError: If the server address is invalid or not reachable.

        An example to get details of a prediction (id=1) from airt service:

        ```python
        Prediction.details(id=1)
        ```
        """
        details = Client.get_data(relative_url=f"/prediction/{id}")

        additional_cols = ["model_id", "datasource_id", "error"]

        columns = Prediction._get_columns() + additional_cols

        details_df = pd.DataFrame(details, index=[0])[columns]

        return add_ready_column(details_df)

    @staticmethod
    def delete(id: int) -> pd.DataFrame:
        """Delete a prediction from airt service

        Args:
            id: The prediction id in airt service.

        Returns:
            A pandas DataFrame encapsulating the details of the deleted prediction id.

        Raises:
            ConnectionError: If the server address is invalid or not reachable.

        An example to delete a prediction (id=1) from airt service:

        ```python
        Prediction.delete(id=1)
        ```
        """
        predictions = Client.delete_data(relative_url=f"/prediction/{id}")

        columns = Prediction._get_columns()

        predictions_df = pd.DataFrame(predictions, index=[0])[columns]

        return add_ready_column(predictions_df)

    def to_pandas(self) -> pd.DataFrame:
        raise NotImplementedError()

    def push(self, data_source: "airt.components.datasource.DataSource"):  # type: ignore
        raise NotImplementedError()


# Cell


@patch
def to_pandas(self: Prediction) -> pd.DataFrame:
    """A function to convert the predicted results into a Pandas DataFrame object.

    Returns:
        A Pandas DataFrame that contains the prediction results from the model.

    Raises:
        ValueError: If the input parameters to the API are invalid.
        ConnectionError: If the server address is invalid or not reachable.

    The below example illustrates the usage of to_pandas function:

    ```python
    from datetime import timedelta

    Client.get_token()
    data_source_s3 = DataSource.s3(
        uri="s3://test-airt-service/ecommerce_behavior"
    )
    data_source_s3.pull().progress_bar()
    model = data_source_s3.train(
        client_column="user_id",
        target_column="event_type",
        target="*purchase",
        predict_after=timedelta(hours=3),
    )

    predictions = model.predict()
    predictions.to_pandas()
    ```
    """
    response = Client.get_data(relative_url=f"/prediction/{self.prediction_id}/pandas")
    keys = list(response.keys())
    keys.remove("Score")
    index_name = keys[0]
    return (
        pd.DataFrame(response)
        .set_index(index_name)
        .sort_values("Score", ascending=False)
    )


# Cell


@patch
def push(self: Prediction, data_source: "airt.components.datasource.DataSource") -> ProgressStatus:  # type: ignore
    """A function to push the prediction results into the target data source.

    For more information on the supported data sources, please refer to the documentation on `DataSource` class

    Args:
        data_source: An instance of the `DataSource` class that encapsulates the data.

    Returns:
        An instance of `ProgressStatus` class. `ProgressStatus` is a base class for querying status of a remote operation. For more information
        please refer to `ProgressStatus` class documentation.

    Raises:
        ValueError: If the input parameters to the API are invalid.
        ConnectionError: If the server address is invalid or not reachable.


    The below example illustrates pushing the prediction results to a database:

    ```python
    from datetime import timedelta

    Client.get_token()

    data_source_s3 = DataSource.s3(
        uri="s3://test-airt-service/ecommerce_behavior"
    )
    data_source_s3.pull().progress_bar()

    model = data_source_s3.train(
        client_column="user_id",
        target_column="event_type",
        target="*purchase",
        predict_after=timedelta(hours=3),
    )

    predictions = model.predict()

    data_source_pred = DataSource.s3(
        uri="s3://target-bucket"
    )

    progress = predictions.push(data_source_pred)
    progress.progress_bar()
    ```
    """
    response = Client.post_data(
        relative_url=f"/prediction/{self.prediction_id}/push",
        data=dict(data_id=data_source.id),
    )
    return ProgressStatus(relative_url=f"/prediction/push/{int(response['id'])}")



# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/CLI_Model.ipynb (unless otherwise specified).

__all__ = ["logger"]

# Cell

from typing import *

# Internal Cell

import os
import typer
from typer import echo
from tabulate import tabulate

from . import helper
from ..logger import get_logger, set_level

# Internal Cell

app = typer.Typer()

# Cell

logger = get_logger(__name__)

# Internal Cell


@app.command()
@helper.requires_auth_token
def ls(
    offset: int = typer.Option(
        0,
        "--offset",
        "-o",
        help="The number of records to offset at the beginning of the models list. If None, then the default value 0 will be used.",
    ),
    limit: int = typer.Option(
        100,
        "--limit",
        "-l",
        help="The maximum number of records to return from the server. If None, then the default value 100 will be used.",
    ),
    disabled: bool = typer.Option(
        False,
        "--disabled",
        help="If set to True, then only the deleted models will be displayed. Else, the default value "
        "False will be used to display only the list of active models.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output only ids of model separated by space",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> None:
    """List available models."""

    from ..client import Model

    df = Model.ls(offset=offset, limit=limit, disabled=disabled)

    df["created"] = helper.humanize_date(df["created"])

    if quiet:
        ids = df["id"].astype(str).to_list()
        typer.echo("\n".join(ids))
    else:
        typer.echo(tabulate(df, headers="keys", tablefmt="plain", showindex=False))


# Internal Cell


@app.command()
@helper.requires_auth_token
def details(
    id: int = typer.Argument(
        ...,
        help="The model ID in the airt service.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> None:
    """Return metadata of the given model."""

    from ..client import Model

    df = Model.details(id=int(id))

    df["created"] = helper.humanize_date(df["created"])

    typer.echo(tabulate(df, headers="keys", tablefmt="plain", showindex=False))


# Internal Cell


@app.command()
@helper.requires_auth_token
def rm(
    id: int = typer.Argument(
        ...,
        help="The model ID in the airt service.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output the deleted model id only.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> None:
    """Remove the model from airt service."""

    from ..client import Model

    df = Model.delete(id=int(id))

    df["created"] = helper.humanize_date(df["created"])

    if quiet:
        typer.echo(df.iloc[0]["id"])
    else:
        typer.echo(tabulate(df, headers="keys", tablefmt="plain", showindex=False))


# Internal Cell


@app.command()
@helper.requires_auth_token
def predict(
    data_id: int = typer.Option(
        ...,
        "--data",
        help="The data ID in the airt service.",
    ),
    model_id: int = typer.Option(
        ...,
        "--model",
        help="The model ID in the airt service.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output the prediction id only.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> None:
    """Make predictions using the trained model."""

    from ..client import Model

    model = Model(model_id=int(model_id))

    prediction = model.predict(data_id=int(data_id))

    if quiet:
        prediction.wait()

        typer.echo(f"{prediction.prediction_id}")
    else:
        typer.echo(f"Running predictions for prediction id: {prediction.prediction_id}")
        prediction.progress_bar()


# Internal Cell


@app.command()
@helper.requires_auth_token
def evaluate(
    id: int = typer.Argument(
        ...,
        help="The model ID in the airt service.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> None:
    """Evaluate the performance of the trained model."""

    from ..client import Model

    model = Model(model_id=int(id))

    df = model.evaluate()

    typer.echo(tabulate(df, headers="keys", tablefmt="plain", showindex=False))



# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/Client.ipynb (unless otherwise specified).

__all__ = [
    "Client",
    "DataSource",
    "ProgressStatus",
    "Model",
    "Prediction",
    "User",
    "APIKey",
]

# Cell

from .components.client import Client as _Client
from .components.datasource import DataSource as _DataSource
from .components.progress_status import ProgressStatus as _ProgressStatus
from .components.model import Model as _Model
from .components.prediction import Prediction as _Prediction
from .components.user import User as _User
from .components.api_key import APIKey as _APIKey


Client = _Client
DataSource = _DataSource
ProgressStatus = _ProgressStatus
Model = _Model
Prediction = _Prediction
User = _User
APIKey = _APIKey

for cls in [Client, DataSource, ProgressStatus, Model, Prediction, User, APIKey]:
    cls.__module__ = "airt.client"



# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/CLI_Version.ipynb (unless otherwise specified).

__all__ = ["logger"]

# Cell

from typing import *

# Internal Cell

import typer
import logging
from tabulate import tabulate

from ..client import Client
from ..logger import get_logger, set_level

# Internal Cell

app = typer.Typer()

# Cell

logger = get_logger(__name__)

# Internal Cell


def version() -> None:
    """Get the version of airt client and airt service."""
    df = Client.version().to_frame()

    typer.echo(tabulate(df, headers="keys", tablefmt="plain"))


# Internal Cell


app.command()(version)



# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/CLI_User.ipynb (unless otherwise specified).

__all__ = ["logger"]

# Cell

from typing import *

# Internal Cell

import os

import typer
from typer import echo
from tabulate import tabulate
import datetime as dt
import pandas as pd

from ..client import Client
from . import helper
from ..logger import get_logger, set_level

# Internal Cell

app = typer.Typer()

# Cell

logger = get_logger(__name__)

# Internal Cell


@app.command()
@helper.requires_auth_token
def create(
    username: str = typer.Option(
        ...,
        "--username",
        "-un",
        help="The username for the new user. If the passed username is already available in airt service,"
        "then an error message will be diplayed indicating the same.",
    ),
    first_name: str = typer.Option(
        ...,
        "--first_name",
        "-fn",
        help="The first name for the new user.",
    ),
    last_name: str = typer.Option(
        ...,
        "--last_name",
        "-ln",
        help="The last name for the new user.",
    ),
    email: str = typer.Option(
        ...,
        "--email",
        "-e",
        help="The email for the new user. If the passed email is already available in airt service, then an error message will be diplayed indicating the same.",
    ),
    password: str = typer.Option(
        ...,
        "--password",
        "-p",
        help="The password for the new user.",
    ),
    super_user: bool = typer.Option(
        False,
        "--super_user",
        "-su",
        help="If set to True, then the new user will have super user privilages. If None, then the default value False will be used.",
    ),
    quota: bool = typer.Option(
        False,
        "--quota",
        "-qt",
        help="If set to True, then the user will have access to all the routes. If None, then the default value False will be used and the user will have access only to `DataSource` routes.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output user id only.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> None:
    """Create a new user in airt service."""

    from ..client import User

    df = User.create(
        username=username,
        first_name=first_name,
        last_name=last_name,
        email=email,
        quota=quota,
        super_user=super_user,
        password=password,
    )

    df["created"] = helper.humanize_date(df["created"])

    if quiet:
        typer.echo(df.iloc[0]["id"])
    else:
        typer.echo(tabulate(df, headers="keys", tablefmt="plain", showindex=False))


# Internal Cell


@app.command()
@helper.requires_auth_token
def ls(
    offset: int = typer.Option(
        0,
        "--offset",
        "-o",
        help="The number of records to offset at the beginning of the users list. If None, then the default value 0 will be used.",
    ),
    limit: int = typer.Option(
        100,
        "--limit",
        "-l",
        help="The maximum number of records to return from the server. If None, then the default value 100 will be used.",
    ),
    disabled: bool = typer.Option(
        False,
        "--disabled",
        help="If set to True, then both active and disabled users will be displayed. If None, "
        "then the default value False will be used which will display only the list of active users.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output only ids of users separated by space",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> None:
    """Display the list of available users."""

    from ..client import User

    df = User.ls(offset=offset, limit=limit, disabled=disabled)

    df["created"] = helper.humanize_date(df["created"])

    if quiet:
        ids = df["id"].astype(str).to_list()
        typer.echo("\n".join(ids))
    else:
        typer.echo(tabulate(df, headers="keys", tablefmt="plain", showindex=False))


# Internal Cell


@app.command()
@helper.requires_auth_token
def disable(
    id: int = typer.Argument(
        ...,
        help="The user id to disabled in airt service.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output user id only.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> None:
    """Disable a user in airt service."""

    from ..client import User

    df = User.disable(user_id=int(id))

    df["created"] = helper.humanize_date(df["created"])

    if quiet:
        ids = df["id"].astype(str).to_list()
        typer.echo("\n".join(ids))
    else:
        typer.echo(tabulate(df, headers="keys", tablefmt="plain", showindex=False))


# Internal Cell


@app.command()
@helper.requires_auth_token
def enable(
    id: int = typer.Argument(
        ...,
        help="The id of the disabled user in airt service.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output user id only.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> None:
    """Enable a disabled user in airt service."""

    from ..client import User

    df = User.enable(user_id=int(id))

    df["created"] = helper.humanize_date(df["created"])

    if quiet:
        ids = df["id"].astype(str).to_list()
        typer.echo("\n".join(ids))
    else:
        typer.echo(tabulate(df, headers="keys", tablefmt="plain", showindex=False))


# Internal Cell


@app.command()
@helper.requires_auth_token
def update(
    user_id: int = typer.Option(
        ...,
        "--user_id",
        "-id",
        help="The id of the user in airt service.",
    ),
    username: str = typer.Option(
        None,
        "--username",
        "-un",
        help="New username for the user_id in airt service. If None (default value), then the username won't be updated for the user.",
    ),
    first_name: str = typer.Option(
        None,
        "--first_name",
        "-fn",
        help="New first_name for the user_id in airt service. If None (default value), then the first_name won't be updated for the user.",
    ),
    last_name: str = typer.Option(
        None,
        "--last_name",
        "-ln",
        help="New last_name for the user_id in airt service. If None (default value), then the last_name won't be updated for the user.",
    ),
    email: str = typer.Option(
        None,
        "--email",
        "-e",
        help="New email for the user_id in airt service. If None (default value), then the email won't be updated for the user.",
    ),
    password: str = typer.Option(
        None,
        "--password",
        "-p",
        help="New password for the user_id in airt service. If None (default value), then the password won't be updated for the user.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output user id only.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> None:
    """Update user details in airt service."""

    from ..client import User

    df = User.update(
        user_id=user_id,
        username=username,
        first_name=first_name,
        last_name=last_name,
        email=email,
        password=password,
    )

    df["created"] = helper.humanize_date(df["created"])

    if quiet:
        typer.echo(df.iloc[0]["id"])
    else:
        typer.echo(tabulate(df, headers="keys", tablefmt="plain", showindex=False))



# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/API_Client.ipynb (unless otherwise specified).

__all__ = ["Client"]

# Cell

from typing import *

# Internal Cell

import os

import pandas as pd
from fastcore.foundation import patch

import airt
from ..logger import get_logger, set_level
from ..helper import get_data, post_data, delete_data, get_base_url

# Internal Cell

logger = get_logger(__name__)

# Internal Cell


def _get_credentials(
    username: Optional[str] = None, password: Optional[str] = None
) -> Tuple[(str, str)]:
    """Returns the value for username and password.

    If username is **None**, retrive the value from AIRT_SERVICE_USERNAME environment variable.
    If password is **None**, retrive the value from AIRT_SERVICE_PASSWORD environment variable.

    Args:
        username: Username for your developer account.
        password: Password for your developer account.

    Returns:
        The values for username and password as a tuple.

    Raises:
        Key Error, if the environment variables are not set.
    """

    username = (
        username if username is not None else os.environ.get("AIRT_SERVICE_USERNAME")
    )

    password = (
        password if password is not None else os.environ.get("AIRT_SERVICE_PASSWORD")
    )

    if not username and not password:
        raise KeyError(
            f"The username and password are neither passed as parameters nor set in the environment variables "
            f"`AIRT_SERVICE_USERNAME` and `AIRT_SERVICE_PASSWORD`."
        )

    elif not username:
        raise KeyError(
            f"The username is neither passed as parameter nor set in the environment variable AIRT_SERVICE_USERNAME."
        )

    elif not password:
        raise KeyError(
            f"The password is neither passed as parameter nor set in the environment variable AIRT_SERVICE_PASSWORD."
        )

    return (username, password)


# Cell


class Client:
    """A class for authenticating and accessing the airt service.

    Before you can use the service, you must acquire a username and password for your developer account. Please contact us by email **info@airt.ai** to get one.

    The username, password, and server address can either be passed explicitly while calling the `get_token` method or stored
    in environment variables **AIRT_SERVICE_USERNAME**, **AIRT_SERVICE_PASSWORD**, and **AIRT_SERVER_URL**.

    Upon successful authentication, the airt services will be available.
    """

    server: Optional[str] = None
    auth_token: Optional[str] = None

    def __init__(self, server: str, auth_token: str):
        Client.server = server
        Client.auth_token = auth_token

    @classmethod
    def get_token(
        cls, *, username: str = None, password: str = None, server: str = None
    ):
        """Get application token for airt service from a username/password pair.

        This method calls the AIRT services for validating the supplied username and password. Upon successful
        authentication, an auth token will be returned and will be implicitly used in all the subsequent interactions with the server.

        Args:
            username: Username for your developer account. If not set (default value **None**), it will try to
                use the value from environment variable **AIRT_SERVICE_USERNAME**.
            password: Password for your developer account. If not set (default value **None**), it will try to
                use the value from environment variable **AIRT_SERVICE_PASSWORD**.
            server: Server address used to connect to. If not set (default value **None**), it will try to
                use the value from environment variable **AIRT_SERVER_URL**. If the variable is not set as well,
                then the default public server will be used. You should leave this to default value unless you
                are running your own server (please contact us for that possibility by email info@airt.ai).

        Raises:
            ValueException: If the username/password pair does not match the one for sevice hosted at server.
            ConnectionError: If the server address is invalid or not reachable.
            KeyError: if username or password are None and there are not stored in environment variables

        """
        cls.server = get_base_url(server)

        username, password = _get_credentials(username, password)

        response = post_data(
            url=f"{cls.server}/token",
            data=dict(username=username, password=password),
            token=None,
        )

        cls.auth_token = response["access_token"]

    @staticmethod
    def version() -> pd.Series:
        """Return the version of airt client and airt service

        Returns:
            A pandas series containing the version of both airt client and airt service

        Raises:
            ConnectionError: If the server address is invalid or not reachable.

        An example to get the version of airt-client and airt-service:

        ```python
        Client.version()
        ```
        """

        response = Client.get_data(relative_url=f"/version")

        version = {
            "airt-client": airt.__version__,
            "airt-service": response["airt_service"],
        }

        return pd.Series(version, name="Version")

    @classmethod
    def _get_server_url_and_token(cls) -> Tuple[Optional[str], Optional[str]]:
        """Fetch the server URL and the auth token.

        Returns:
            A tuple containing server URL and auth token.
        """

        cls.server = get_base_url(cls.server)

        return cls.server, cls.auth_token

    @classmethod
    def post_data(
        cls, relative_url: str, data: Optional[Dict[str, Any]]
    ) -> Dict[str, Any]:
        """Make a POST request.

        This method will implicitly add the server base URL and the token for every request.

        Args:
            relative_url: The relative URL of the server's API endpoint.
            data: A Dictionary object to send in the body of the POST request.

        Returns:
            Response body as a dictionary.

        Raises:
            ConnectionError: If the server is not reachable.
            ValueError: If the response code is not in range of 200 - 399.
        """

        server, auth_token = Client._get_server_url_and_token()

        return post_data(
            url=f"{server}{relative_url}",
            data=data,
            token=auth_token,
        )

    @classmethod
    def get_data(cls, relative_url: str) -> Any:
        """Make a GET request.

        This method will implicitly add the server base URL and the token for every request.

        Args:
            relative_url: The relative URL of the API endpoint.

        Returns:
            A dictionary that encapsulates the response body.

        Raises:
            ConnectionError: If the server is not reachable.
            ValueError: If the response code is not in range of 200 - 399.
        """

        server, auth_token = Client._get_server_url_and_token()

        return get_data(url=f"{server}{relative_url}", token=auth_token)

    @classmethod
    def delete_data(cls, relative_url: str) -> Dict[str, Any]:
        """Make a DELETE request.

        This method will implicitly add the server base URL and the token for every request.

        Args:
            relative_url: The relative URL of the API endpoint.

        Returns:
            A dictionary that encapsulates the response body.

        Raises:
            ConnectionError: If the server is not reachable.
            ValueError: If the response code is not in range of 200 - 399.
        """

        server, auth_token = Client._get_server_url_and_token()

        return delete_data(url=f"{server}{relative_url}", token=auth_token)



# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/API_Helper.ipynb (unless otherwise specified).

__all__ = [
    "ensure_is_instance",
    "get_base_url",
    "post_data",
    "get_data",
    "delete_data",
    "add_ready_column",
    "generate_df",
    "get_tag_str",
]

# Cell

from typing import *

# Internal Cell

import os
import requests

import pandas as pd

# Cell


def ensure_is_instance(o: Any, cls: Type):
    """A function to check if the object argument is an instance of the class argument.

    Args:
        o: A python object for which the instance needs to be checked.
        cls: The expected instance of the object argument.

    Raises:
        A TypeError if the object is not an instance of the class type.
    """
    if not isinstance(o, cls):
        raise TypeError(
            f"The parameter must be a {cls} type, but got `{type(o).__name__}`"
        )


# Internal Cell


def _get_json(response: requests.Response) -> Any:
    """A function to validate the status of the response object.

    Args:
        response: The response object that encapsulates the server's response.

    Returns:
        A dictionary of the response body.

    Raises:
        ValueError: If the response code is not in range of 200 - 399.
    """
    if response:
        return response.json()
    else:
        raise ValueError(response.json()["detail"])


# Cell


def get_base_url(server: Optional[str]) -> str:
    """Return the base URL for the airt server.

    If the server value is `None`, retrive the value from the environment variable `AIRT_SERVER_URL`.
    If the variable is not set as well, then the default public server will be used.
    """

    env_airt_service_url = "AIRT_SERVER_URL"
    airt_prod_server_url = "https://api.airt.ai"

    return (
        server
        if server is not None
        else os.environ.get(env_airt_service_url, airt_prod_server_url)
    )


# Cell


def post_data(
    url: str,
    data: Optional[Dict[str, Any]],
    token: Optional[str],
) -> Dict[str, Any]:
    """A function to send a POST request.

    Args:
        url: The URL of the server to which the request needs to be sent.
        data: A Dictionary object to send in the body of the POST request.
        token: The unique auth token for the client, obtained via calling the `Client.get_token()` method.
            Set it to `None` in `Client.get_token()` to obtain the token.

    Returns:
        A dictionary that encapsulates the response body.

    Raises:
        ConnectionError: If the server is not reachable.
        ValueError: If the response code is not in range of 200 - 399.
    """
    if token is not None:
        headers = {"Authorization": f"Bearer {token}"}
        response = requests.post(url, json=data, headers=headers)
    else:
        response = requests.post(url, data=data)
    return _get_json(response)


# Cell


def get_data(url: str, token: Optional[str]) -> Any:
    """Send a GET request.

    Args:
        url: The URL of the server to which the request needs to be sent.
        token: The unique auth token for the client, obtained via calling the `Client.get_token()` method.

    Returns:
        A dictionary that encapsulates the response body.

    Raises:
        ConnectionError: If the server is not reachable.
        ValueError: If the response code is not in range of 200 - 399.
    """
    headers = {"Authorization": f"Bearer {token}"}
    response = requests.get(url, headers=headers)
    return _get_json(response)


# Cell


def delete_data(url: str, token: Optional[str]) -> Dict[str, Any]:
    """Send a DELETE request.

    Args:
        url: The URL of the server to which the request needs to be sent.
        token: The unique auth token for the client, obtained via calling the `Client.get_token()` method.

    Returns:
        A dictionary that encapsulates the response body.

    Raises:
        ConnectionError: If the server is not reachable.
        ValueError: If the response code is not in range of 200 - 399.
    """
    headers = {"Authorization": f"Bearer {token}"}
    response = requests.delete(url, headers=headers)
    return _get_json(response)


# Cell


def add_ready_column(df: pd.DataFrame) -> pd.DataFrame:
    """Add ready column to the DataFrame

    Args:
        df: A pandas DataFrame with completed_steps and total_steps columns

    Returns:
        A pandas DataFrame with ready column
    """

    df["ready"] = df["completed_steps"] == df["total_steps"]
    return df.drop(columns=["completed_steps", "total_steps"])


# Cell


def generate_df(items: Dict[str, Any], columns: list) -> pd.DataFrame:
    """Generate a DataFrame based on the items length

    Args:
        items: A list encapsulating the response from an API endpoint that needs to be converted into a DataFrame.
        columns: A list of columns names to be included in the DataFrame.

    Returns:
        A DataFrame with a shape of (items, columns), if the length of the items is > 0, otherwise an empty DataFrame with only columns names.
    """

    if len(items) > 0:
        df = pd.DataFrame(items)[columns]
    else:
        df = pd.DataFrame({c: [] for c in columns})

    return df


# Cell


def get_tag_str(tags: list) -> str:
    """Get tag names as string seperated by comma from the response

    Args:
        tags: The tags list from the response

    Returns:
        The tag names as string seperated by comma. If the tags list is empty, <none> will be returned.
    """

    if len(tags) == 0:
        return "<none>"

    return ", ".join([t["name"] for t in tags])



# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/API_Model.ipynb (unless otherwise specified).

__all__ = ["Model"]

# Cell

from typing import *

# Internal Cell

import os

import pandas as pd
from fastcore.foundation import patch

from ..logger import get_logger, set_level
from ..helper import get_data, post_data, delete_data, add_ready_column, generate_df

from .client import Client
from .progress_status import ProgressStatus
from .prediction import Prediction

# Internal Cell

logger = get_logger(__name__)

# Cell


class Model(ProgressStatus):
    """A class to query the status of the model training, evaluation, and prediction on the remote server.

    A `Model` class is instantiated automatically when the `DataSource.train` method of the `DataSource` class is called.
    Currently, it is the only way to instantiate the Model class.

    The model is trained on the connected data and tries to predict a specific event in the future.
    For the model training and prediction, we assume the input data includes the following:

    - a column identifying a client client_column (person, car, business, etc.),

    - a column specifying a type of event we will try to predict target_column (buy, checkout, etc.),

    - a timestamp column specifying the time of an occurred event.

    Along with the above mandatory fields, each row in the data might have additional columns of int,
    category, float, or datetime type and they will be used to make predictions more accurate.

    Finally, we need to know how much ahead we wish to make predictions for. That lead time varies
    widely from application to application and can be in minutes for a webshop or even several weeks for a banking product such as a loan.

    As always, the model training and prediction will happen asynchronously and can take a few hours
    based on the size of your dataset.

    The training status can either be viewed interactively by calling the `progress_bar` method or
    non-interactively by calling the `is_ready` method on the model instance which returns a boolean
    representing the training completion status.

    """

    def __init__(self, model_id: int):
        """Constructs a new `Model` instance

        Warning:
            Do not construct this object directly by calling the constructor, please use
            `DataSource.train()` function instead.

        Args:
            model_id: ID of the model in the airt service
        """
        self.model_id = model_id
        ProgressStatus.__init__(self, relative_url=f"/model/{self.model_id}")

    @staticmethod
    def _get_columns() -> list:
        """Returns the list of columns to be added to the DataFrame."""

        return ["id", "created", "total_steps", "completed_steps"]

    @staticmethod
    def ls(
        offset: int = 0,
        limit: int = 100,
        disabled: bool = False,
        completed: bool = False,
    ) -> pd.DataFrame:
        """Display the list of available models in airt service.

        Args:
            offset: The number of rows to offset at the beginning of the models
                list from the server.If **None**, then the default value **0** will be used.
            limit: The maximum number of rows to return from the server. If **None**,
                then the default value **100** will be used.
            disabled: If set to **True**, then only the deleted models will be displayed.
                Else, the default value **False** will be used to display only the list
                of active models.
            completed: If set to **True**, then only the models that are successfully trained
                in airt server will be displayed. Else, the default value **False** will be used
                to display all the models including the ones that are yet to finish training.

        Returns:
            A pandas dataframe with the list of available models.

        Raises:
            ConnectionError: If the server address is invalid or not reachable.

        An example to list the available models:

        ```python
        Model.ls()
        ```
        """
        models = Client.get_data(
            relative_url=f"/model/?disabled={disabled}&completed={completed}&offset={offset}&limit={limit}"
        )

        columns = Model._get_columns()

        models_df = generate_df(models, columns)

        return add_ready_column(models_df)

    @staticmethod
    def details(id: int) -> pd.DataFrame:
        """Return details of a model

        Args:
            id: The id of the model in the airt service.

        Returns:
            A pandas dataframe with the details of the model.

        Raises:
            ConnectionError: If the server address is invalid or not reachable.

        An example to get details of a model (id=1) from airt service:

        ```python
        Model.details(id=1)
        ```
        """

        details = Client.get_data(relative_url=f"/model/{id}")

        additional_cols = [
            "datasource_id",
            "user_id",
            "client_column",
            "target_column",
            "target",
            "predict_after",
            "timestamp_column",
            "error",
            "disabled",
        ]

        columns = Model._get_columns() + additional_cols

        details_df = pd.DataFrame(details, index=[0])[columns]

        return add_ready_column(details_df)

    @staticmethod
    def delete(id: int) -> pd.DataFrame:
        """Delete a model from airt service

        Args:
            id: The model id in airt service.

        Returns:
            A pandas DataFrame encapsulating the details of the deleted model id.

        Raises:
            ConnectionError: If the server address is invalid or not reachable.

        An example to delete a model (id=1) from airt service:

        ```python
        Model.delete(id=1)
        ```
        """

        models = Client.delete_data(relative_url=f"/model/{id}")

        columns = Model._get_columns()

        models_df = pd.DataFrame(models, index=[0])[columns]

        return add_ready_column(models_df)

    def predict(self, data_id: Optional[int]) -> "airt.client.Prediction":  # type: ignore
        raise NotImplementedError()

    def evaluate(self) -> pd.DataFrame:
        raise NotImplementedError()


# Cell


@patch
def evaluate(self: Model) -> pd.DataFrame:
    """A function to evaluate the performance of the trained model.

    This function returns the performance metrics like accuracy, precision, and recall. Currently,
    the function returns only the above-mentioned metrics and we plan to add more performance metrics in the future.

    Returns:
        A pandas Series that has the performance metrics of the trained model.

    Raises:
        ValueError: If the input parameters to the API are invalid.
        ConnectionError: If the server address is invalid or not reachable.
    """
    model_evaluate = Client.get_data(relative_url=f"/model/{self.model_id}/evaluate")
    return pd.DataFrame(dict(model_evaluate), index=[0]).T.rename(columns={0: "eval"})


# Cell


@patch
def predict(self: Model, data_id: Optional[int] = 0) -> Prediction:
    """A function that uses the trained model and makes predictions.

    As always, this function is asynchronous and can take a few hours based on the size of your dataset. The status of
    the model prediction can be viewed interactively by calling the `progress_bar` method available on the returned object.
    For more information, please check the documentation of `DataSource`

    Args:
        data_id: The data id for running the predictions. If the value is not passed then the data id used for
            training will be used for prediction aswell.
    Returns:
        An instance of the `Prediction` class. For more information on the methods that are available in
        the returned object, please check the documentation of the `Prediction` class

    Raises:
        ValueError: If the input parameters to the API are invalid.
        ConnectionError: If the server address is invalid or not reachable.
    """

    request_body = dict(data_id=data_id) if data_id else None

    response = Client.post_data(
        relative_url=f"/model/{self.model_id}/predict", data=request_body
    )

    return Prediction(
        prediction_id=response["id"], datasource_id=response["datasource_id"]
    )



# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/API_User.ipynb (unless otherwise specified).

__all__ = ["User"]

# Cell

from typing import *

# Internal Cell

import os

import pandas as pd
from fastcore.foundation import patch

from ..logger import get_logger, set_level
from ..helper import get_data, post_data, delete_data, generate_df

from .client import Client

# Internal Cell

logger = get_logger(__name__)

# Cell


class User:
    """A class to encapsulate all the user management routes

    All the user management routes are only accessable to super users. If a non-super user tried to access
    the user management routes they will get a permission error.
    """

    @staticmethod
    def create(
        *,
        username: str,
        first_name: str,
        last_name: str,
        email: str,
        password: str,
        super_user: bool = False,
        quota: bool = False,
    ) -> pd.DataFrame:
        """A method to create a new user in airt service

        Args:
            username: The username for the new user. If the passed username is already available in airt service, then an error message
                will be diplayed indicating the same.
            first_name: The first name for the new user.
            last_name: The last name for the new user.
            email: The email for the new user. If the passed email is already available in airt service, then an error message
                will be diplayed indicating the same.
            password: The password for the new user.
            super_user: If set to **True**, then the new user will have super user privilages. If **None**, then the default value **False** will be used.
            quota: If set to True, then the user will have access to all the routes. If **None**, then the default value **False** will be used and the user
                will have access only to `DataSource` routes.

        Returns:
                A pandas dataframe with the details of the newly created user

        Raises:
            ConnectionError: If the server address is invalid or not reachable.
            ValueError: If the username or email is already present in the airt service.

        An minimal example for calling the `create` method:

        ```python
            new_user = User.create(
                username="example_username",
                first_name="example_first_name",
                last_name="example_last_name",
                email="example@example.com",
                password="example_password",
            )
        ```
        """
        req_data = dict(
            username=username,
            first_name=first_name,
            last_name=last_name,
            email=email,
            quota=quota,
            super_user=super_user,
            password=password,
        )

        response = Client.post_data(relative_url=f"/user/", data=req_data)

        columns = User._get_columns()

        return pd.DataFrame(response, index=[0])[columns]

    @classmethod
    def _get_columns(cls) -> list:
        """Returns the list of columns to be added to the DataFrame."""

        return [
            "id",
            "username",
            "super_user",
            "disabled",
            "created",
            "quota",
            "first_name",
            "last_name",
            "email",
        ]

    @staticmethod
    def ls(
        offset: int = 0,
        limit: int = 100,
        disabled: bool = False,
    ) -> pd.DataFrame:
        """Display the list of available users.

        Args:
            offset: The number of rows to offset at the beginning of the users
                list from the server.If **None**, then the default value **0** will be used.
            limit: The maximum number of rows to return from the server. If **None**,
                then the default value **100** will be used.
            disabled: If set to **True**, then both active and disabled users will be displayed. If **None**,
                then the default value **False** will be used which will display only the list of active users.

        Returns:
            A pandas dataframe with the list of available users.

        Raises:
            ConnectionError: If the server address is invalid or not reachable.

        An example to list the available users:

        ```python
        User.ls()
        ```
        """
        users = Client.get_data(
            relative_url=f"/user/?disabled={disabled}&offset={offset}&limit={limit}"
        )

        columns = User._get_columns()

        return generate_df(users, columns)

    @staticmethod
    def disable(user_id: int) -> pd.DataFrame:
        """Disable a user in airt service.

        Args:
            user_id: The user id to disabled in airt service.

        Returns:
            A pandas dataframe with the details of the disabled user.

        Raises:
            ConnectionError: If the server address is invalid or not reachable.

        An example to disable a user (id=1) in airt service:

        ```python
        User.disable(user_id=1)
        ```
        """

        response = Client.delete_data(relative_url=f"/user/{user_id}")

        columns = User._get_columns()

        return pd.DataFrame(response, index=[0])[columns]

    @staticmethod
    def enable(user_id: int) -> pd.DataFrame:
        """Enable a disabled user in airt service.

        Args:
            user_id: The id of the disabled user in airt service.

        Returns:
            A pandas dataframe with the details of the enabled user.

        Raises:
            ConnectionError: If the server address is invalid or not reachable.

        An example to enable a disabled user (id=1) in airt service:

        ```python
        User.enable(user_id=1)
        ```
        """

        response = Client.get_data(relative_url=f"/user/{user_id}/enable")

        columns = User._get_columns()

        return pd.DataFrame(response, index=[0])[columns]

    @staticmethod
    def update(
        user_id: int,
        username: Optional[str] = None,
        first_name: Optional[str] = None,
        last_name: Optional[str] = None,
        email: Optional[str] = None,
        password: Optional[str] = None,
    ) -> pd.DataFrame:
        """Update user details in airt service.

        Args:
            user_id: The id of the user in airt service.
            username: New **username** for the user_id in airt service. If **None** (default value), then the username won't be updated for the user.
            first_name: New **first_name** for the user_id in airt service. If **None** (default value), then the first_name won't be updated for the user.
            last_name: New **last_name** for the user_id in airt service. If **None** (default value), then the last_name won't be updated for the user.
            email: New **email** for the user_id in airt service. If **None** (default value), then the email won't be updated for the user.
            password: New **password** for the user_id in airt service. If **None** (default value), then the password won't be updated for the user.

        Returns:
            A pandas dataframe with the updated user details.

        Raises:
            ConnectionError: If the server address is invalid or not reachable.

        An example to update the username for a user (id=1) in airt service:

        ```python
        User.update(user_id=1, username="new_username")
        ```
        """

        body = dict(
            username=username,
            first_name=first_name,
            last_name=last_name,
            email=email,
            password=password,
        )

        response = Client.post_data(relative_url=f"/user/{user_id}/update", data=body)

        columns = User._get_columns()

        return pd.DataFrame(response, index=[0])[columns]



# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/ActivateTestingEnvironment.ipynb (unless otherwise specified).

__all__ = ["activated"]

# Cell


def activated() -> bool:
    return True


# Internal Cell

from ..logger import get_logger, set_level, supress_timestamps  # type: ignore

supress_timestamps()

# Cell

import logging

# Internal Cell

logger = get_logger(__name__)

set_level(logging.INFO)

logger.info("Testing environment activated.")



# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/CLI_Key.ipynb (unless otherwise specified).

__all__ = ["logger"]

# Cell

from typing import *

# Internal Cell

import os
import typer
from typer import echo
from tabulate import tabulate
import pandas as pd

from ..client import Client
from . import helper
from ..logger import get_logger, set_level

# Internal Cell

app = typer.Typer(
    help="""Set of commands for managing the API keys in airt service.

        Both the API key and the token can be used for accessing the airt services. However, there is a slight difference in generating and managing the two.

        For generating the API key, you first need to get the token. Please refer to the documentation of the airt token command for generating one.

        Once successfully authenticated, you can create any number of API keys and set an expiration date for each key individually. Also, you can use the other
        methods available in the APIKey class to list, revoke the keys at any time.

        You can now set the API key in the environment variable AIRT_SERVICE_TOKEN and start accessing airt services with it.""",
)

# Cell

logger = get_logger(__name__)

# Internal Cell


@app.command()
@helper.requires_auth_token
def create(
    name: str = typer.Option(..., "--name", "-n", help="Name for the new API key"),
    expiry: int = typer.Option(
        ...,
        "--expiry",
        "-e",
        help="The validity for the newly created API key in the number of days",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output access token only.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> None:
    """Create a new API key in airt service"""

    from ..client import APIKey

    response = APIKey.create(name=name, expiry=expiry)

    if quiet:
        typer.echo(response["access_token"])
    else:
        typer.echo(f"Access Token: {response['access_token']}")


# Internal Cell


@app.command()
@helper.requires_auth_token
def ls(
    offset: int = typer.Option(
        0,
        "--offset",
        "-o",
        help="The number of records to offset at the beginning of the API keys list. If None, then the default value 0 will be used.",
    ),
    limit: int = typer.Option(
        100,
        "--limit",
        "-l",
        help="The maximum number of records to return from the server. If None, then the default value 100 will be used.",
    ),
    include_disabled: bool = typer.Option(
        False,
        "--disabled",
        help="If set to True, then the disabled API keys will also be included in the result.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output only ids of API keys separated by space",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> None:
    """List available API keys."""

    from ..client import APIKey

    df = APIKey.ls(offset=offset, limit=limit, include_disabled=include_disabled)

    df["created"] = helper.humanize_date(df["created"])
    df["expiry"] = helper.humanize_date(df["expiry"])

    if quiet:
        ids = df["id"].astype(str).to_list()
        typer.echo("\n".join(ids))
    else:
        typer.echo(tabulate(df, headers="keys", tablefmt="plain", showindex=False))


# Internal Cell


@app.command()
@helper.requires_auth_token
def details(
    id: int = typer.Argument(
        ...,
        help="The API Key ID in the airt service.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> None:
    """Return details of the given API Key."""

    from ..client import APIKey

    df = APIKey.details(api_key_id=int(id))

    df["created"] = helper.humanize_date(df["created"])
    df["expiry"] = helper.humanize_date(df["expiry"])

    typer.echo(tabulate(df, headers="keys", tablefmt="plain", showindex=False))


# Internal Cell


@app.command()
@helper.requires_auth_token
def rm(
    id: int = typer.Argument(
        ...,
        help="The API Key ID in the airt service.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output the deleted API key id only.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> None:
    """Remove the API key from airt service."""

    from ..client import APIKey

    df = APIKey.delete(api_key_id=int(id))

    df["created"] = helper.humanize_date(df["created"])
    df["expiry"] = helper.humanize_date(df["expiry"])

    if quiet:
        typer.echo(df.iloc[0]["id"])
    else:
        typer.echo(tabulate(df, headers="keys", tablefmt="plain", showindex=False))



# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/CLI.ipynb (unless otherwise specified).

__all__ = ["logger", "app"]

# Cell

from typing import *
import logging

# Internal Cell

import typer

from . import ds, model, pred, user, api_key
from .token import token
from .version import version
from ..logger import get_logger, set_level

# Cell

logger = get_logger(__name__)

set_level(logging.WARNING)

# Cell

app = typer.Typer(help="airt CLI for accessing airt services.")

# Internal Cell

# token root command

app.command()(token)

# Internal Cell

# Datasource root command

app.add_typer(
    ds.app,
    name="ds",
    short_help="A command for encapsulating the data from sources like CSV files, databases, or AWS S3 buckets.",
    help="A class for encapsulating the data from sources like CSV files, databases, or AWS S3 buckets. "
    "Currently, we support reading and pushing the data from a CSV file, a MySql database, and an "
    "AWS S3 bucket in the Parquet file format. We plan to add other databases and storage "
    "mediums in the future.",
)

# Internal Cell

# Model root command

app.add_typer(
    model.app,
    name="model",
    short_help="Query the status of the model training, evaluation, and prediction on the remote server.",
    help="Query the status of the model training, evaluation, and prediction on the remote server.",
)

# Internal Cell

# Pred root command

app.add_typer(
    pred.app,
    name="pred",
    short_help="Run predictions on the data source.",
    help="Run predictions on the data source.",
)

# Internal Cell

# User root command

app.add_typer(
    user.app,
    name="user",
    short_help="User management commands",
    help="User management commands. All the user management routes are only accessable to super users. "
    "If a non-super user tried to access the user management commands a permission error will be displayed.",
)

# Internal Cell

# API Keys root command

app.add_typer(
    api_key.app,
    name="api-key",
    short_help="API keys management commands",
    help="""Set of commands for managing the API keys in airt service.

        Both the API key and the token can be used for accessing the airt services. However, there is a slight difference in generating and managing the two.

        For generating the API key, you first need to get the token. Please refer to the documentation of the airt token command for generating one.

        Once successfully authenticated, you can create any number of API keys and set an expiration date for each key individually. Also, you can use the other
        methods available in the APIKey class to list, revoke the keys at any time.

        You can now set the API key in the environment variable AIRT_SERVICE_TOKEN and start accessing airt services with it.""",
)

# Internal Cell

# version root command

app.command()(version)
